/*! \mainpage wxTest:  A C++ Unit Test Framework

 *



      <h1>Overview</h1>

      <p>wxTest is a test framework designed in the tradition of the xUnit

      family.&nbsp; It's helpful for the development of automated test suites for

      C++ code.&nbsp; wxTest consists of an interface specification between <i>

      tests</i> and <i> test- runners</i>, in the form of abstract virtual C++ classes.&nbsp;

      This interface facilitates complete binary decoupling between the tests

      and the test modules.&nbsp; Tests are packaged in

      separate modules (.DLLs, .shlbs, .sos, etc) and are completely

      decoupled from the test runner.</p>

      <p>Despite the 'wx' prefix, wxTest can be used to test any C++ code, not

      just code that uses wxWindows.&nbsp; The interface between tests and test runners is pure

      C++ and has no dependency to wxWindows.&nbsp; wxTest includes two

      implementations of test runners: a GUI and a text-only version.&nbsp;  </p>

      <h2>&nbsp;</h2>

      <h1>Features</h1>

      <p>For the initial 1.0 release, only Win32, Macintosh (Carbon), and GTK/Linux 

      are targeted&nbsp; Not all of these features are currently implemented on 

      all platforms:</p>

      <p>Hierarchal Unit Tests.&nbsp; Test cases are related hierarchally.&nbsp; 

      Tests are contained in a test suite.&nbsp; Test suites may in turn be 

      contained in other test suites. </p>

      <p>Binary Decoupled Unit Tests:&nbsp; Unit tests are packaged into 

      separate dynamic libraries called &quot;<i>test- modules</i>.&quot;&nbsp; A test- 

      module exports a single function &quot;<code>GetWXTest</code>&quot; which retrieves 

      a pointer to the test suite implemented by the test module.&nbsp; Test 

      modules aren't linked to the test runner.&nbsp; At runtime the test runner 

      loads the test modules dynamically and calls the <code>GetWXTest</code> 

      function to obtain a pointer to the modules test suites.</p>

      <p>Unit-Test and Test-Runner Interfaces:&nbsp; The interface between tests 

      and test-runners is defined by pure abstract C++ classes.&nbsp; This 

      enables binary-decoupling in a cross platform manner.</p>

      <p>Textual Test Runner:&nbsp; Implements the &quot;runner&quot; side of the 

      unit-test / test-runner interface.&nbsp; It loads tests modules an 

      executes unit tests, sending the output to stout.</p>

      <p>GUI Test Runner:&nbsp; Implements the &quot;runner&quot; side of the unit-test / 

      test-runner interface.&nbsp; It loads tests modules an executes unit tests 

      and displays the test tree.&nbsp; You can interactively browse through 

      tests and execute them, or set , sending the output to stdout.&nbsp; The 

      GUI Test Runner has additional features, such as the ability to check 

      textual output, and running test cases in multiple threads simultaneously 

      to test for thread safety.</p>

      <h2>&nbsp;</h2>

      <h1>Building wxTest</h1>

<p>You must first install and build wxWindows.&nbsp; See

<a href="http://www.wxwindows.org">http://www.wxwindows.org</a> for the latest 

version and more information.</p>

<h3>Win32: DevStudio 6</h3>

<p>Configure your build environment for wxWindows' lib and include paths:&nbsp; 

(From the menu, select <b>Tool </b>-- <b>Options</b>, and select the <b>

Directories</b> tab.&nbsp; Add the include and lib paths for wxWindows if 

they're not there already.).</p>

<p>In the folder <code style="font-family: Courier New; font-weight: bold">wxTest/win32</code>, open the workspace file 

<code style="font-family: Courier New; font-weight: bold">wxTest.dsw</code>.&nbsp; 

Select from the menu <b>Build </b>--<b> Batch Build</b>, make sure everything every target is 

checked, and click the '<b>Build</b>' button.</p>

<p>Once it's built you can execute the wxTestRunnerGUI target to run the sample 

tests.</p>

<h3>Macintosh:&nbsp; CodeWarrior 6 / 7</h3>

<p>Macintosh CodeWarrior support is not yet available.</p>

<h3>Linux:</h3>

<p>Linux support is not yet available.</p>

<h2>&nbsp;</h2>

<h2>Creating Unit Tests</h2>

<p>Create a new test-module.&nbsp; The easiest way to do this is to copy the 

entire contents of the 'sample' subdirectory to a new location, and to rename 

everything from 'Sample' to whatever you'd like to call it.&nbsp; You'll also 

have to adjust the include and lib paths. I'm sorry there isn't a more 

automagical way of doing this, but there really isn't that much code there.&nbsp; </p>

<p>Create a new class derived from wxTestCase.&nbsp; ( Use SampleTests as a 

guide. )</p>

<p>Implement the Setup and TearDown functions to initialize or cleanup anything 

needed by your tests.&nbsp; These are called before and after each call to the 

individual tests.</p>

<p>Implement one or more test functions.&nbsp; The must have the signature&nbsp;&nbsp; 

void TestFunctionName( wxITestRunner * pTestRunner, ThreadId threadId ).&nbsp; 

This is where you put your actual test code.&nbsp; The wxITestRunner pointer is the interface by which you test communicates with 

the test runner.&nbsp; The threadId tells you which thread the thread in which 

the test function is being called.</p>

<p>Create a Suite method that returns a test-suite that can be used to call each 

of the test methods.&nbsp; Use the wxTestCaller template class to populate a 

wxTestSuite object.&nbsp; (See the sample for an example.)</p>

<p>Create the <code>GetWXTest<code> function using the WXTEST_EXPORT macro so it is exported 

from the shared library.&nbsp; (See SampleTests.cpp for an example.)</p>

      <p>





 */



