#ifndef __WXTESTBASE_H__

#define __WXTESTBASE_H__



#include <wxTest/ITest.h>



WXTEST_BEGIN_NAMESPACE



/** 

 * @defgroup TestLib  TestLib:  Useful base implementations of the test side of the test/test-runner interface.

 */



/**

 * A base implementation of the wxITest interface.

 * @ingroup TestLib

 */

class wxTestBase : public wxITest

{

public:

	wxTestBase( const char* pszDescription, wxITest* pParent = 0 );

	virtual ~wxTestBase();



	/**

	 *  Destroys and deallocates the test

	 */ 

	virtual void Delete();



	/**

	 *  Retreive a human readable description of this test.

	 *

	 *  @return A null terminated string that is the description of this test.

	 */ 

	virtual const char* GetDescription();



	/**

	 * Retreive the current TestState of this test.

	 *

	 * @return The current TestState of this test.

	 */ 

	virtual TestState GetState();



	virtual void SetState( TestState newState );



	/**

	 * Begin executing this test.

	 *

	 * @param pResult The result object that will recieve the results of this test.

	 */ 

	virtual void Run( wxITestRunner* pResult ) {}



	/**

	 * Begin executing this test in a multi-threaded enviroment.

	 *

	 * @param pResult The result object that will recieve the results of this test.

	 * @param nThreadId The thread identifer of the thread in which this test is running.

	 */ 

	virtual void Run( wxITestRunner* pResult, long unsigned nThreadId ) {}



	/**

	 * Determine if this test is intended to be thread-safe.  This useful for test runners

	 * that may want to run the test in multiple threads simultaneously to test for 

	 * resource conflicts and deadlocks.

	 */ 

	virtual bool IsThreadSafe();



	/**

	 * Retreive the number of sub-tests for this test.

	 * 

	 */ 

	virtual int GetNumSubTests() { return 0; }



	/**

	 * Retreive the child-tests of this test.

	 *

	 * @param pOutTests A pointer to a buffer that will recieve pointers to the sub-tests.

	 */ 

	virtual void GetSubTests( wxITest ** pOutTests ) {}





	/**

	 * Retreive the parent of this test.

	 *

	 * @return A pointer to the parent test of this test, or NULL if this test has no parent.

	 */ 

	virtual const wxITest* GetParent() const;



	/**

	 * Retreive the parent of this test.

	 *

	 * @return A pointer to the parent test of this test, or NULL if this test has no parent.

	 */ 

	virtual wxITest* GetParent() ;



	/**

	 * Assign a new parent to this test.

	 *

	 * @param pParent The test that is to be the parent of this test.

	 */ 

	virtual void SetParent( wxITest* pParent ) ;



	/**

	 * Set up this object's state before actually executing any tests.

	 */

	virtual void Setup();



	/**

	 * Clean up anything that was allocated in Setup.

	 */

	virtual void TearDown();





protected:

	typedef long unsigned ThreadId;



	char* m_pszDescription;

	TestState m_state;

	wxITest* m_pParent;



};



WXTEST_END_NAMESPACE



#endif //__WXTESTBASE_H__



