/////////////////////////////////////////////////////////////////////////////

// Name:        TestCaseMultiThreaded.h

// Purpose:     Add multi-threaded testing to the TestCase implementation.

// Author:      Marty Saxton

// Modified by:

// Created:     10/27/2001

// RCS-ID:      $Id: TestCaseMultiThreaded.h,v 1.1.1.1 2001/11/10 19:00:43 thundercow Exp $

// Copyright:   (c) Marty Saxton, Thundercow Software

// Licence:     wxWindows license

/////////////////////////////////////////////////////////////////////////////



#ifndef __WXTESTCASE_H__

#define __WXTESTCASE_H__

#include <wxTest/TestCase.h>



//WXTEST_BEGIN_NAMESPACE



/**

 * Provides additional implementation of a test specifically for a test case.

 * @ingroup TestLib

 */

class wxTestCaseMultiThreaded : public wxTestCase

{

public:

	wxTestCaseMultiThreaded( const char* pszDescription, wxITest* pParent = 0 );

//	virtual ~wxTestCaseMultiThreaded();



	/**

	 * Retreive the number of sub-tests for this test.

	 * 

	 */ 

	virtual int GetNumSubTests();



	/**

	 * Retreive the child-tests of this test.

	 *

	 * @param pOutTests A pointer to a buffer that will recieve pointers to the sub-tests.

	 */ 

	virtual void GetSubTests( wxITest ** pOutTests );



	/**

	 * Begin executing this test.

	 *

	 * @param pResult The result object that will recieve the results of this test.

	 */ 

	virtual void Run( wxITestRunner* pResult );



	/**

	 * Begin executing this test in a multi-threaded enviroment.

	 *

	 * @param pResult The result object that will recieve the results of this test.

	 * @param nThreadId The thread identifer of the thread in which this test is running.

	 */ 

	virtual void Run( wxITestRunner* pResult, long unsigned nThreadId );



protected:

	virtual void RunUnitTest( wxITestRunner* pResult, ThreadId nThreadId ) = 0;



};



WXTEST_END_NAMESPACE



#endif //__WXTESTCASE_H__

