/////////////////////////////////////////////////////////////////////////////

// Name:        TestRunner.h

// Purpose:     Provide default implementation for the test runner interface wxITestRunner

// Author:      Marty Saxton

// Modified by:

// Created:     10/27/2001

// RCS-ID:      $Id: TestRunner.h,v 1.1.1.1 2001/11/10 19:00:43 thundercow Exp $

// Copyright:   (c) Marty Saxton, Thundercow Software

// Licence:     wxWindows license

/////////////////////////////////////////////////////////////////////////////



#include <wx/wxprec.h>

#ifndef WX_PRECOMP

#	include <wx/wx.h>

#endif



#include <wxTest/TestSuite.h>

#include <wxTest/ITestRunner.h>

#include <stdio.h>



struct wxTestModule;

class wxDynamicLibrary;



struct wxTestModule

{

	wxDynamicLibrary * m_pLib;

	wxString m_strModuleFileName;

	wxITest* m_pTest;

	typedef wxITest* (*GetWXTestsFunc)();

	GetWXTestsFunc m_pTestFunc;

};





class wxTestRunner : public wxITestRunner

{

public:

	wxTestRunner();

	virtual ~wxTestRunner();





	/**

	 * Captures "standard out" which is informational text output from tests.

	 * If tests create any "standard out" output, then it must be identical each

	 * time the test is run.

	 */

	virtual void wxtstdout( const char* psz, ... );



	/**

	 * Called by the test framework before a unit test is run.

	 */

	virtual void BeginTestCase( wxITest* pTest );



	/**

	 * Called by the test framework after a unit test is run.

	 */

	virtual void EndTestCase( wxITest* pTest );



	virtual void OnTestFailure( wxITest* pTest, const char* pszMessage, const char* pszFile, unsigned long nLineNo ); 



	virtual wxString GetQualifiedTestDescription( wxITest* pTest );



	virtual void LoadTests();



	virtual void OnFoundTestModule( wxTestModule& pTest );



	virtual void OnTestAdded( wxITest* pTest );



	virtual void SpawnThreads( wxITest* );

	virtual void BeginTests();





	virtual void OnTestOutput( wxITest* pTest, const char* pszOutput );

	virtual void OnErrorOutput( const char* pszOutput );



	virtual int GetNumTestsRun();



	virtual int GetNumFailures();



	wxITest* GetTestRoot();



	virtual void GetOutputSubdir( wxITest* pTest, char* pszBuffer, bool bCreate );



	virtual wxString GetOutputSubdir(); 

	virtual void SetOutputSubDir( wxString strNewOutputSubDir );





protected:



	virtual void OpenFiles( wxITest* pTest );

	virtual void CloseFiles();

	FILE* m_stdout;

	wxITest *m_pCurrentTest;





	wxTestModule* m_modules;

	int m_nModuleCount;

	int m_nModuleCapacity;



	int m_nFailures;

	int m_nFailuresThisTestCase;

	int m_nTestsRun;

	wxString m_strLibDir;

	wxString m_strOutputDir;

	wxTestSuite* m_pRootTest;

};



class TestRunnerThread;



class wxTestRunnerMT : public wxTestRunner

{

public:

	wxTestRunnerMT();

	virtual ~wxTestRunnerMT();

	void SetNumThreads( int nThreads );

	virtual void OnThreadTerminated( TestRunnerThread* pThread );

	int m_nThreads;

	virtual void SpawnThreads( wxITest* );

protected:

	TestRunnerThread** m_threads;

};

