/////////////////////////////////////////////////////////////////////////////

// Name:        wxMacInitializeTerminate.cpp

// Purpose:     

// Author:      Marty Saxton

// Modified by:

// Created:     10/27/2001

// RCS-ID:      $Id: TestSuite.h,v 1.1.1.1 2001/11/10 19:00:43 thundercow Exp $

// Copyright:   (c) Marty Saxton, Thundercow Software

// Licence:     wxWindows license

/////////////////////////////////////////////////////////////////////////////



#ifndef __WXTESTSUITE_H__

#define __WXTESTSUITE_H__



#include <wxTest/ITest.h>



/**

 * An implementation of the wxITest interface that aggregates multiple sub-tests.

 * @ingroup TestLib

 */

class wxTestSuite : public wxITest

{

public:

	wxTestSuite( const char* pszDescription );



	/**

	 *  Destroys and deallocates the test

	 */ 

	virtual void Delete();





	virtual ~wxTestSuite();



	/**

	 *  Retreive a human readable description of this test.

	 *  @return A null terminated string that is the description of this test.

	 */ 

	virtual const char* GetDescription();



	/**

	 * Retreive the current TestState of this test.

	 * @return The current TestState of this test.

	 */ 

	virtual TestState GetState();



	virtual void SetState( TestState newState );



	/**

	 * Begin executing this test.

	 * @param pResult The result object that will recieve the results of this test.

	 */ 

	virtual void Run( wxITestRunner* pResult );



	/**

	 * Begin executing this test in a multi-threaded enviroment.

	 *

	 * @param pResult The result object that will recieve the results of this test.

	 * @param nThreadId The thread identifer of the thread in which this test is running.

	 */ 

	virtual void Run( wxITestRunner* pResult, long unsigned nThreadId );



	/**

	 * Determine if this test is intended to be thread-safe.  This useful for test runners

	 * that may want to run the test in multiple threads simultaneously to test for 

	 * resource conflicts and deadlocks.

	 */ 

	virtual bool IsThreadSafe();



	/**

	 * Retreive the number of sub-tests for this test.

	 * 

	 */ 

	virtual int GetNumSubTests();



	/**

	 * Retreive the child-tests of this test.

	 * @param pOutTests A pointer to a buffer that will recieve pointers to the sub-tests.

	 */ 

	virtual void GetSubTests( wxITest ** pOutTests );



	/**

	 * Retreive the parent of this test.

	 * @return A pointer to the parent test of this test, or NULL if this test has no parent.

	 */ 

	virtual const wxITest* GetParent() const;



	/**

	 * Retreive the parent of this test.

	 * @return A pointer to the parent test of this test, or NULL if this test has no parent.

	 */ 

	virtual wxITest* GetParent();



	/**

	 * Assign a new parent to this test.

	 *

	 * @param pParent The test that is to be the parent of this test.

	 */ 

	virtual void SetParent( wxITest* pParent );



	/**

	 * Adds a test to this wxTestSuite.

	 * @param pTest The test to be added to this wxTestSuite.

	 */

	virtual void AddTest( wxITest* pTest );



private:

	char* m_pszDescription;

	TestState m_state;

	wxITest** m_tests;

	int m_nTestCount;

	int m_nTestCapacity;

	wxITest* m_pParent;

};

#endif //__WXTESTSUITE_H__

