/////////////////////////////////////////////////////////////////////////////

// Name:        TestTests.h

// Purpose:     

// Author:      Marty Saxton

// Modified by:

// Created:     10/27/2001

// RCS-ID:      $Id: TestTests.cpp,v 1.1.1.1 2001/11/10 19:00:44 thundercow Exp $

// Copyright:   (c) Marty Saxton, Thundercow Software

// Licence:     wxWindows license

/////////////////////////////////////////////////////////////////////////////



#include "wx/wxprec.h"

#ifndef WX_PRECOMP

    #include "wx/wx.h"

#endif



#include <wxTest/ITestRunner.h>



#include <wxTest/TestCase.h>

#include <wxTest/TestSuite.h>

#include <wxTest/TestCaller.h>

#include <wxTest/TestRunner.h>

#include <wx/app.h>



#ifdef _MSC_VER

#include <windows.h>

#endif



#include "../../Sample/wxTestSample.h"



class wxTestRunnerTest : public wxTestRunner

{

public:

	virtual void OnTestFailure( wxITest* pTest, const char* pszMessage, const char* pszFile, unsigned long nLineNo )

	{

		wxTestRunner::OnTestFailure( pTest,  pszMessage, pszFile, nLineNo );

		m_pTest = pTest;

		strcpy( m_szMessage, pszMessage );

		strcpy( m_szFile , pszFile );

		m_nLineNo = nLineNo;

	}



	wxITest* m_pTest;

	char m_szMessage[ 1024];

	char m_szFile [ 1024 ];

	unsigned long m_nLineNo;





	virtual void OnTestOutput( wxITest* pTest, const char* pszOutput ) { m_strOutput += pszOutput; }

	wxString m_strOutput;



	virtual void OpenFiles( wxITest* ) {}

	virtual void CloseFiles()		  {}



};



class wxTestTest : public wxTestCase

{

public:

	wxTestTest( const char* pszTestName ) : wxTestCase( pszTestName ) {}



	void Pass( wxITestRunner* , long unsigned )

	{

		wxTestCaller< SampleTests > test( "Test Pass", SampleTests::Pass );

		wxTestRunnerTest runner;

		test.Run( &runner );

		WXTASSERT( runner.GetNumTestsRun() == 1 && runner.GetNumFailures() == 0 );

	}



	void Fail( wxITestRunner* , long unsigned )

	{

		wxTestCaller< SampleTests > test( "Test Fail", SampleTests::Fail );

		wxTestRunnerTest runner;

		test.Run( &runner );



		WXTASSERT( runner.GetNumTestsRun() == 1 );

		WXTASSERT( runner.GetNumFailures() == 1 );

		WXTASSERT( runner.m_pTest == &test );

		WXTASSERT( !strcmp( runner.m_szMessage, ""  ) );

		WXTASSERT( strlen( runner.m_szFile ) );

		WXTASSERT( runner.m_nLineNo );

	}



	void FailWithMessage( wxITestRunner* , long unsigned )

	{

		wxTestCaller< SampleTests > test( "Test Fail", SampleTests::FailWithMessage );

		wxTestRunnerTest runner;

		test.Run( &runner );



		WXTASSERT( runner.GetNumTestsRun() == 1 );

		WXTASSERT( runner.GetNumFailures() == 1 );

		WXTASSERT( runner.m_pTest == &test );

		WXTASSERT( !strcmp( runner.m_szMessage, FAILURE_MESSAGE  ) );

		WXTASSERT( strlen( runner.m_szFile ) );

		WXTASSERT( runner.m_nLineNo );

	}



	void Error( wxITestRunner* , long unsigned )

	{

		wxTestCaller< SampleTests > test( "Test Error", SampleTests::Error );

		wxTestRunnerTest runner;

		test.Run( &runner );



		WXTASSERT( runner.GetNumTestsRun() == 1 );

		WXTASSERT( runner.GetNumFailures() == 1 );

		WXTASSERT( runner.m_pTest == &test );

		WXTASSERT( !strcmp( runner.m_szMessage, "ERROR"  ) );

		WXTASSERT( !strcmp( runner.m_szFile, "<UNKNOWN>" ) );

		WXTASSERT( !runner.m_nLineNo );

	}



	void Output( wxITestRunner* , long unsigned )

	{

		wxTestCaller< SampleTests > test( "Test Output", SampleTests::Output );

		wxTestRunnerTest runner;

		test.Run( &runner );



		WXTASSERT( runner.GetNumTestsRun() == 1 );

		WXTASSERT( runner.GetNumFailures() == 0 );

		WXTASSERT( runner.m_strOutput == "This is a test of normal test output." );

	}



	static wxITest* Suite()

	{

		wxTestSuite* pSuite = new wxTestSuite("TestTest");

		pSuite->AddTest ( new wxTestCaller<wxTestTest> ("Pass", Pass ) );

		pSuite->AddTest ( new wxTestCaller<wxTestTest> ("Fail", Fail ) );

		pSuite->AddTest ( new wxTestCaller<wxTestTest> ("FailWithMessage", FailWithMessage ) );

		pSuite->AddTest ( new wxTestCaller<wxTestTest> ("Output", Output ) );



		// It appears that only VC++ catches errors in catch(...).

#		ifdef __WXMSW__

			pSuite->AddTest ( new wxTestCaller<wxTestTest> ("Error", Error ) );

#		endif



		return pSuite;

	}



};



WXTEST_EXPORT wxITest* GetWXTest()

{

	return wxTestTest::Suite();

}





class wxTestTestApp : public wxApp

{

    virtual bool OnInit()

	{

		wxApp::OnInit();

		return true;

	}

};



IMPLEMENT_APP( wxTestTestApp )



#ifdef _MSC_VER



BOOL WINAPI DllMain( HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved )

{

	if ( fdwReason == DLL_PROCESS_ATTACH )

		wxEntry( hinstDLL, NULL, "", 1 , 0);

	else if ( fdwReason == DLL_PROCESS_DETACH )

		wxApp::CleanUp();

	return TRUE;

}



#endif

