/////////////////////////////////////////////////////////////////////////////

// Name:        wxTestSuite.h

// Purpose:     

// Author:      Marty Saxton

// Modified by:

// Created:     10/27/2001

// RCS-ID:      $Id: wxTestSuite.cpp,v 1.1.1.1 2001/11/10 19:00:45 thundercow Exp $

// Copyright:   (c) Marty Saxton, Thundercow Software

// Licence:     wxWindows license

/////////////////////////////////////////////////////////////////////////////



#include "../include/wxTest/TestSuite.h"



wxTestSuite::wxTestSuite( const char* pszDescription ) : 

	m_nTestCount( 0 ),

	m_nTestCapacity( 0 ),

	m_tests( 0 ),

	m_pParent( 0 ),

	m_state( unknown )

{

	const char* pEnd = pszDescription;

	while ( *pEnd )

		pEnd++;



	m_pszDescription = new char[ pEnd - pszDescription + 1 ];

	char* p = m_pszDescription;

	const char* q = pszDescription;

	while ( *q )

		*p++ = *q++;

	*p = 0;

}







wxTestSuite::~wxTestSuite()

{

	int nTest;

	for ( nTest = 0; nTest < m_nTestCount; ++nTest )

		m_tests[ nTest ] ->Delete();





	delete [] m_pszDescription;

	delete [] m_tests;

}



void wxTestSuite::Delete()

{

	delete this;

}



const char* wxTestSuite::GetDescription()

{

	return m_pszDescription;

}



wxTestSuite::TestState wxTestSuite::GetState()

{

	return m_state;

}



void wxTestSuite::SetState( TestState newState )

{

	m_state = newState;

}



void wxTestSuite::Run( wxITestRunner* pResult, long unsigned nThreadId )

{

}



void wxTestSuite::Run( wxITestRunner* pResult )

{

	wxITest** ppTest = m_tests, ** ppEnd = m_tests + m_nTestCount;

	while ( ppTest != ppEnd ) 

		(*ppTest++)->Run( pResult );

}



bool wxTestSuite::IsThreadSafe()

{

	wxITest ** ppTest = m_tests, ** ppEnd = m_tests + m_nTestCount;

	while ( ppTest != ppEnd ) 

		if ( !(*ppTest)->IsThreadSafe() )

			return false;

	return true;

}



int wxTestSuite::GetNumSubTests()

{

	return m_nTestCount;

}



void wxTestSuite::GetSubTests( wxITest ** pOutTests )

{

	wxITest ** ppTest = m_tests, ** ppEnd = m_tests + m_nTestCount, ** ppOut = pOutTests;

	while ( ppTest != ppEnd )

		*ppOut++ = *ppTest++;

}



const wxITest* wxTestSuite::GetParent() const

{

	return m_pParent;

}



wxITest* wxTestSuite::GetParent()

{

	return m_pParent;

}



void wxTestSuite::SetParent( wxITest* pParent )

{

	m_pParent = pParent;

}



void wxTestSuite::AddTest( wxITest* pTest )

{

	pTest->SetParent( this );

	if ( m_nTestCount == m_nTestCapacity )

	{

		if ( m_nTestCapacity == 0 )

		{

			m_nTestCapacity	= 8;

			m_tests = new wxITest*[ m_nTestCapacity ];

		}

		else

		{

			m_nTestCapacity *= 2;

			wxITest ** pTempTests = new wxITest*[ m_nTestCapacity ];

			wxITest ** pSrc = m_tests;

			wxITest ** pEndSrc = m_tests + m_nTestCount;

			wxITest ** pDst = pTempTests;

			while ( pSrc < pEndSrc )

				*pDst++ = *pSrc++;

			wxITest ** pTempDeleteMe = m_tests;

			m_tests = pTempTests;

			delete [] pTempDeleteMe;

		}

	}

	m_tests[ m_nTestCount++ ] = pTest;

}



