/////////////////////////////////////////////////////////////////////////////

// Name:        wxTestRunner.cpp

// Purpose:     

// Author:      Marty Saxton

// Modified by:

// Created:     10/27/2001

// RCS-ID:      $Id: wxTestRunner.cpp,v 1.1.1.1 2001/11/10 19:00:45 thundercow Exp $

// Copyright:   (c) Marty Saxton, Thundercow Software

// Licence:     wxWindows license

/////////////////////////////////////////////////////////////////////////////



#include <wxTest/TestRunner.h>



#ifdef __WXMSW__

#include <direct.h>

#endif // __WXMSW__



#ifdef __WXGTK__

//#include <sys/stat.h>

#endif





#include <wx/dynlib.h>

#include <wx/dir.h>



#if   defined __WXMSW__

#include <malloc.h>

#elif defined __WXMAC__

#include <alloca.h>

#endif



static const unsigned PATHNAME_SIZE = 1024;



bool CreateFullPath( const char* pszPath )

{

	wxString wxstrInitialDir = wxGetCwd();

	wxString wxCurDir = wxstrInitialDir;

	bool bDirError = false;



	char* pszTempPath = new char[ strlen( pszPath ) + 1 ];

	strcpy( pszTempPath, pszPath );

	if ( pszTempPath[ strlen( pszTempPath )-1 ] == wxFILE_SEP_PATH )

		pszTempPath[strlen( pszTempPath )-1] = 0;

	char *pTok = pszTempPath;



	wxString strCurrentOutputFileName  = "";



	for (;;) 

	{

		char *pEndTok = strchr( pTok, wxFILE_SEP_PATH );

		if ( pEndTok )

			*pEndTok = 0;



		// special handling for absolute paths

		if ( pTok == pszTempPath && wxIsAbsolutePath( pszPath ) )

		{





	#ifdef __WXMSW__

			if ( wxCurDir.size() && wxCurDir[ wxCurDir.size()-1 ] != ':' )

			{

				wxSetWorkingDirectory( "\\" );

				wxSetWorkingDirectory( pTok );

				wxCurDir = "";

			}

			else

			{

	#endif

				wxCurDir = pTok;

				wxSetWorkingDirectory( ( wxCurDir + wxFILE_SEP_PATH ) .c_str() );







	#ifdef __WXMSW__

			}

	#endif





			if ( !pEndTok )

				break;

			pTok = pEndTok + 1;

			continue;



		}



		strCurrentOutputFileName  += pTok;

		

			

		if ( !wxDirExists( pTok ) ) 

			wxMkdir( pTok );



		if ( !wxDirExists( pTok ) ) 

		{

			wxLogError( "warning:  cannot create output dir %s, no output will be collected", strCurrentOutputFileName .c_str() );

			bDirError = true;

			break;

		}

		

		wxString wxstrNewCurrentDir =  wxCurDir + wxFILE_SEP_PATH + strCurrentOutputFileName;

		wxSetWorkingDirectory( wxstrNewCurrentDir );

		strCurrentOutputFileName  += wxFILE_SEP_PATH;

		if ( !pEndTok )

			break;

		pTok = pEndTok + 1;

	}



	delete [] pszTempPath;



	wxSetWorkingDirectory( wxstrInitialDir );

	return !bDirError;

}



void wxTestRunner::wxtstdout( const char* psz, ... )

{

	int nSizeGuess = strlen( psz ) * 2;

	for (;;)

	{

		va_list va;

		va_start( va, psz );     

		char * p = (char*)alloca( nSizeGuess );

		if ( vsprintf( p, psz, va ) >= 0 )

		{

			OnTestOutput( m_pCurrentTest, p );

			break;

		}

		nSizeGuess *= 2;

	}

}



void wxTestRunner::BeginTestCase( wxITest* pTest )

{

	m_nFailuresThisTestCase = m_nFailures;

	pTest->SetState( wxITest::running );

	m_nTestsRun++;

	m_pCurrentTest = pTest;



	char szPath[PATHNAME_SIZE];

	GetOutputSubdir( pTest, szPath, true );

	char * pEnd = szPath + strlen( szPath );

	*pEnd++ = wxFILE_SEP_PATH;

	strcpy (pEnd, "failures.txt" );

	wxRemoveFile( szPath );

}



void wxTestRunner::EndTestCase( wxITest* pTest )

{

	CloseFiles();

	if ( pTest->GetState() == wxITest::running )

	{

		if ( m_nFailuresThisTestCase != m_nFailures )

			pTest->SetState( wxITest::fail );

		else

			pTest->SetState( wxITest::pass );

	}

}



void wxTestRunner::GetOutputSubdir( wxITest* pTest, char* pszBuffer, bool bCreate )

{

	wxString str = m_strOutputDir;

	str += GetQualifiedTestDescription( pTest );

	strcpy( pszBuffer, str.c_str() );

	if ( bCreate )

		CreateFullPath( pszBuffer );

}



wxString wxTestRunner::GetQualifiedTestDescription( wxITest* pTest )

{

	wxString rv;

	for (;;)

	{

		if ( !pTest->GetParent() )

			break;



		if ( rv.IsEmpty() )

			rv = pTest->GetDescription();

		else

			rv = wxString( pTest->GetDescription() ) + wxFILE_SEP_PATH + rv;



		pTest = pTest->GetParent();

	}



	return rv;

}





void wxTestRunner::OpenFiles( wxITest* pTest )

{					  

	char szPath[PATHNAME_SIZE];

	GetOutputSubdir( pTest, szPath, true );

	char * pEnd = szPath + strlen( szPath );

	*pEnd++ = wxFILE_SEP_PATH;

	strcpy (pEnd, "output.txt" );

	m_stdout = fopen( szPath, "w" );

}

					   

void wxTestRunner::CloseFiles()

{

	if ( m_stdout )

		fclose( m_stdout );

	m_stdout = 0;

}



void wxTestRunner::OnTestFailure( wxITest* pTest, const char* pszMessage, const char* pszFile, unsigned long nLineNo )

{

	char szPath[PATHNAME_SIZE];

	GetOutputSubdir( pTest, szPath, true );

	char * pEnd = szPath + strlen( szPath );

	*pEnd++ = wxFILE_SEP_PATH;

	strcpy (pEnd, "failures.txt" );

	FILE* f = fopen( szPath, "wa" );

	fprintf(f, "FAILURE:  %s:%d %s\n", pszFile, nLineNo, pszMessage );

	fclose(f);





	++m_nFailures;

}





class TestRunnerThread : public wxThread

{

public:

	TestRunnerThread() : m_pTest(0), m_pRunner(0) {}

	~TestRunnerThread() 

	{ 

		m_pRunner->OnThreadTerminated( this ); 

	}



	virtual void SetTest( wxITest* pTest ) { m_pTest = pTest; }

	virtual void SetResult( wxTestRunnerMT* pRunner ) { m_pRunner = pRunner; }



	virtual ExitCode Entry()

	{

		m_pTest->Run( m_pRunner, GetId() );

		return 0;

	}





protected:

	wxITest* m_pTest;

	wxTestRunnerMT * m_pRunner;

};



void wxTestRunnerMT::OnThreadTerminated( TestRunnerThread* pThread )

{

	int nThread;

	for ( nThread = 0; nThread < m_nThreads; ++nThread )

	{

		if ( m_threads[ nThread ] == pThread )

		{

			m_threads[ nThread ] = NULL;

			break;

		}

	}

}





void wxTestRunner::SpawnThreads( wxITest* pTest )

{

	pTest->Run( this, 0 ); 

}









wxTestRunner::wxTestRunner() : 

	m_pRootTest ( new wxTestSuite( "all tests" ) ),

//	m_tests("all tests" ),

	m_modules( 0 ),

	m_nModuleCount( 0 ),

	m_nModuleCapacity( 0 ),

	m_nTestsRun( 0 ),

	m_stdout( 0 ),

	m_nFailures( 0 )

{

	m_strLibDir = wxGetCwd();



	// For Win32 we want the starting directory to be wherever the executible is.

#ifdef __WXMSW__

	char szDir[ _MAX_PATH ], szDrive[ _MAX_PATH ], szPath[ _MAX_PATH];

	DWORD dwResult = GetModuleFileName( NULL, szDir, sizeof( szDir ) ) ;

	if ( dwResult ) {

		_splitpath( szDir, szDrive, szPath, NULL, NULL );

		strcat( szDrive, szPath);

		m_strLibDir = szDrive;

	}

#endif

	m_strOutputDir = m_strLibDir + "TestOuput" + wxFILE_SEP_PATH;

}



wxTestRunner::~wxTestRunner() 

{

	delete m_pRootTest;



	for ( int nModule = 0; nModule < m_nModuleCount; ++nModule )

	{

		delete m_modules[ nModule ].m_pLib;

	}



	delete [] m_modules;

}





static const unsigned WXTEST_MAX_PATH = 1024 * 2;



static const char * const WXTEST_DYNAMIC_LIB_EXT = 

#if defined __WXMSW__

													"*.dll";

#elif defined __WXMAC__

													"*.shlb";

#elif defined __WXGTK__

													"*.so";

#endif





void wxTestRunner::LoadTests()

{

	wxArrayString files;

    size_t s = wxDir::GetAllFiles( m_strLibDir.c_str(), &files, WXTEST_DYNAMIC_LIB_EXT );



	wxTestModule tempTestModule;

	for ( size_t i = 0; i < files.GetCount(); ++i ) {

		wxString wxstr = files[i];



#ifdef __WXMSW__  // supress errors

		UINT nPrevErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);

#endif



		wxDynamicLibrary *pLib = new wxDynamicLibrary( wxstr );



#ifdef __WXMSW__

		SetErrorMode(nPrevErrorMode);

#endif

		

		if ( pLib->IsLoaded() ) {



			bool bLoggingWasEnabled = wxLog::EnableLogging( false );

			wxTestModule::GetWXTestsFunc pGetTest = (wxTestModule::GetWXTestsFunc)pLib->GetSymbol( "GetWXTest");

			wxLog::EnableLogging( bLoggingWasEnabled );



			if ( pGetTest ) {

				tempTestModule.m_pLib = pLib;

				tempTestModule.m_strModuleFileName = wxstr;

				tempTestModule.m_pTest = pGetTest();

				tempTestModule.m_pTestFunc = pGetTest;

				OnFoundTestModule( tempTestModule );



			}

			else {

				delete pLib;

			}

		}

	}

}





wxITest* findSubTest( wxITest* pTest, const char* pszDescription )

{

	int nSubTests = pTest->GetNumSubTests();

	wxITest** tests = (wxITest**)alloca( sizeof(wxITest*) * nSubTests );

	pTest->GetSubTests( tests );



	wxITest** pCurTest = tests, **pEnd = tests + nSubTests;

	while ( pCurTest != pEnd )

	{

		if ( !strcmp( (*pCurTest)->GetDescription(), pszDescription ) )

			return *pCurTest;

		++pCurTest;

	}

	return NULL;

}





void wxTestRunner::OnTestAdded( wxITest*  )

{

}





void wxTestRunner::OnFoundTestModule( wxTestModule& module )

{

	wxITest* pModuleTest = module.m_pTest;



	wxString strModulePath = module.m_strModuleFileName;

	strModulePath = strModulePath.Right( strModulePath.size() - m_strLibDir.size() );

	int nBegin = 0;

	wxString strTemp; 

	wxTestSuite * pTempTest = m_pRootTest;

	for (;;) 

	{

		size_t pos = strModulePath.find( wxFILE_SEP_PATH, nBegin );

		if ( pos == wxString::npos )

			break;

		strTemp = strModulePath.substr( nBegin, pos - nBegin );

		wxTestSuite* pTest = static_cast<wxTestSuite*>( findSubTest( pTempTest, strTemp.c_str() ) );



		if ( !pTest )

		{

			pTest = new wxTestSuite( strTemp.c_str() );

			pTempTest->AddTest( pTest );

			OnTestAdded( pTest );

		}



		nBegin = pos + 1;

		pTempTest = pTest;

	}



	pTempTest->AddTest( pModuleTest );

	if ( m_nModuleCount == m_nModuleCapacity )

	{

		if ( m_nModuleCapacity == 0 )

		{

			m_nModuleCapacity = 8;

			m_modules = new wxTestModule[ m_nModuleCapacity ];

		}

		else

		{

			m_nModuleCapacity *= 2;

			wxTestModule * pTempTests = new wxTestModule[ m_nModuleCapacity ];

			wxTestModule * pSrc = m_modules;

			wxTestModule * pEndSrc = m_modules + m_nModuleCount;

			wxTestModule * pDst = pTempTests;

			while ( pSrc < pEndSrc )

				*pDst++ = *pSrc++;

			wxTestModule * pTempDeleteMe = m_modules;

			m_modules = pTempTests;

			delete [] pTempDeleteMe;

		}

	}

	m_modules[ m_nModuleCount++ ] = module;



	OnTestAdded( module.m_pTest );



}





void wxTestRunner::BeginTests()

{

	for ( int nModule = 0; nModule < m_nModuleCount; ++nModule )

		m_modules[ nModule ].m_pTest->Run( this );



}



int wxTestRunner::GetNumTestsRun()

{

	return m_nTestsRun;

}



int wxTestRunner::GetNumFailures()

{

	return m_nFailures;

}



void wxTestRunner::OnTestOutput( wxITest* pTest,  const char* pszOutput )

{

	if ( !m_stdout )

		OpenFiles( pTest );



	fwrite( pszOutput, sizeof(char), strlen( pszOutput ), m_stdout );

}



void wxTestRunner::OnErrorOutput( const char* pszOutput )

{

}





wxTestRunnerMT::wxTestRunnerMT() :

	m_nThreads( 1 ),

	m_threads( 0 )

{

}



wxTestRunnerMT::~wxTestRunnerMT() 

{

	delete [] m_threads;

}



void wxTestRunnerMT::SpawnThreads( wxITest* pTest )

{

	m_threads = new TestRunnerThread*[ m_nThreads ];



	int nThread;

	for ( nThread = 0; nThread < m_nThreads; ++nThread )

	{

		m_threads[ nThread ] = new TestRunnerThread;

		m_threads[ nThread ]->SetTest( pTest );

		m_threads[ nThread ]->SetResult( this );

		m_threads[ nThread ]->Create();

		m_threads[ nThread ]->Run();

	}



	for (;;) 

	{

		for ( nThread = 0; nThread < m_nThreads; ++nThread )

		{

			if ( m_threads[ nThread ] )

				break;

		}



		if ( nThread == m_nThreads ) 

			break;

			

		wxYield();

	}



	delete m_threads;

	m_threads = NULL;



}



void wxTestRunnerMT::SetNumThreads( int nThreads )

{

	m_nThreads = nThreads;

}



wxITest* wxTestRunner::GetTestRoot()

{

	return m_pRootTest;

}



wxString wxTestRunner::GetOutputSubdir()

{

	return m_strOutputDir;

}



void wxTestRunner::SetOutputSubDir( wxString strNewOutputSubDir )

{

	m_strOutputDir = strNewOutputSubDir;

}



