/////////////////////////////////////////////////////////////////////////////

// Name:        wxTestRunnerFrame.cpp

// Purpose:     

// Author:      Marty Saxton

// Modified by:

// Created:     10/27/2001

// RCS-ID:      $Id: wxTestRunnerFrame.cpp,v 1.1.1.1 2001/11/10 19:00:46 thundercow Exp $

// Copyright:   (c) Marty Saxton, Thundercow Software

// Licence:     wxWindows license

/////////////////////////////////////////////////////////////////////////////



#include "PrecompiledHeaders.h"



#include "wxTestTree.h"

#include "wxTestRunnerFrame.h"

#include "wxTestRunnerGUIApp.h"

  

#include "wx/splitter.h"





//#ifdef _MSC_VER

//#pragma warning (disable : 4786 )

//#include <sys/stat.h>

//#include <malloc.h>

//#endif



//#include "ConfirmResultsDlg.h"

//#include <wx/file.h>

//#ifdef __MWERKS__

//#include <alloca.h>

//#undef wxMkDir

//#define wxMkDir _mkdir

//#include <stat.h>

//#endif

//#include <algorithm>



#if defined(__WXGTK__) || defined(__WXMOTIF__) || defined( __WXMAC__ )

    #include "../../unix/wxTestRunnerGUI.xpm"

#endif





// IDs for the controls and the menu commands

enum

{

    // menu items

    wxTest_QUIT = 1,

	wxTest_OutputClear,

	wxTest_SetOutputDir,

    wxTest_ABOUT

};





BEGIN_EVENT_TABLE(wxTestRunnerFrame, wxFrame)

    EVT_MENU(wxTest_QUIT,  wxTestRunnerFrame::OnQuit)

    EVT_MENU(wxTest_SetOutputDir,  wxTestRunnerFrame::OnSetOutputDir)

    EVT_MENU(wxTest_ABOUT, wxTestRunnerFrame::OnAbout)

END_EVENT_TABLE()



wxTestRunnerFrame::wxTestRunnerFrame(const wxString& title, const wxPoint& pos, const wxSize& size) :

       wxFrame((wxFrame *)NULL, -1, title, pos, size),

	   m_fontMono( 10, wxMODERN, wxNORMAL, wxNORMAL ),

	   m_stdout(0)

{

#ifdef __WXMAC__

    // we need this in order to allow the about menu relocation, since ABOUT is

    // not the default id of the about menu

    wxApp::s_macAboutMenuItemId = wxTest_ABOUT;

#endif



    // set the frame icon

    SetIcon(wxICON(mondrian));



    // create a menu bar

    wxMenu *menuFile = new wxMenu("", wxMENU_TEAROFF);



    // the "About" item should be in the help menu

    wxMenu *helpMenu = new wxMenu;

    helpMenu->Append(wxTest_ABOUT, "&About...\tCtrl-A", "Show about dialog");



    menuFile->Append(wxTest_QUIT, "E&xit\tAlt-X", "Quit this program");



	wxMenu *outputMenu = new wxMenu;

	outputMenu->Append( wxTest_OutputClear, "&Clear All Test Output", "Removes all previous test output." );

	outputMenu->Append( wxTest_SetOutputDir,      "&Set Output Dir", "Set a new output subdirectory." );



    // now append the freshly created menu to the menu bar...

    wxMenuBar *menuBar = new wxMenuBar();

    menuBar->Append(menuFile, "&File");

    menuBar->Append(outputMenu, "&Output");

    menuBar->Append(helpMenu, "&Help");



    // ... and attach this menu bar to the frame

    SetMenuBar(menuBar);



#if wxUSE_STATUSBAR

    // create a status bar just for fun (by default with 1 pane only)

    CreateStatusBar(2);

    SetStatusText("Welcome to BJ-Unit Test Runner!");

#endif // wxUSE_STATUSBAR

	



	// create a splitter that divides the main frame into top (shared by tree and test results), and botton (log window)

	wxSplitterWindow * pMainSplitter= new wxSplitterWindow(this, WINDOWID_SPLITTER_WINDOW, wxDefaultPosition, wxDefaultSize, wxSP_3D | wxSP_LIVE_UPDATE | wxCLIP_CHILDREN );



	// create the log window

    wxGetApp().m_pLogWindow= new wxTextCtrl(	pMainSplitter,					// parent

										WINDOWID_TEST_RESULTS,		// window id

										"",				// value

										wxPoint(0,0),				// pos

										wxSize( 1,1),				// size

							            wxTE_READONLY | wxHSCROLL | wxTE_MULTILINE );

	wxGetApp().m_pLogWindow->SetFont( m_fontMono );

	

	// send wx logging to our log window

	wxLogTextCtrl *plogText = new wxLogTextCtrl( wxGetApp().m_pLogWindow );

	wxLog::SetActiveTarget( plogText );

	

	// create the inner splitter (for the tree and test results winders)

	wxSplitterWindow * pSplitter= new wxSplitterWindow( pMainSplitter, WINDOWID_SPLITTER_WINDOW, wxDefaultPosition, wxDefaultSize, wxSP_3D | wxSP_LIVE_UPDATE | wxCLIP_CHILDREN );



	pMainSplitter->SplitHorizontally(pSplitter, wxGetApp().m_pLogWindow, 500);



	// create the tree control that holds the tests

	wxGetApp().m_pTestTree = new wxTestTree( pSplitter, WINDOWID_TEST_TREE);

	

	// crteate the test results window

    wxGetApp().m_pTestResults = new wxTextCtrl(	pSplitter,					// parent

										WINDOWID_TEST_RESULTS,		// window id

										"",				// value

										wxPoint(0,0),				// pos

										wxSize( 1,1),				// size

							            wxTE_RICH | wxTE_READONLY | wxHSCROLL | wxTE_MULTILINE );



	wxGetApp().m_pTestResults->SetFont( m_fontMono );



	pSplitter->SplitVertically(wxGetApp().m_pTestTree, wxGetApp().m_pTestResults, 200);

}



	   

wxTestRunnerFrame::~wxTestRunnerFrame()

{

}





void wxTestRunnerFrame::OnQuit(wxCommandEvent& WXUNUSED(event))

{

    // TRUE is to force the frame to close

    Close(TRUE);

}



void wxTestRunnerFrame::OnAbout(wxCommandEvent& WXUNUSED(event))

{

    wxString msg;

    msg.Printf( _T("wxTest\n")

                _T("Welcome to %s"), wxVERSION_STRING);



    wxMessageBox(msg, "About wxTest Test Runner", wxOK | wxICON_INFORMATION, this);

}





/*

class TestResultWX : public TestResultDelegator

{

public:

	TestResultWX( TestRunner* pTestRunner ) : 

		TestResultDelegator( pTestRunner )

	{

	}



	virtual void getOutputFileName( const Test* pTest, char* pBuffer, bool bCreate ) const 

	{ 

		m_pRunner->getOutputFileName( pTest, pBuffer, bCreate );

	}



private:

	//std::string m_strTestResult;

};







TestResult* wxTestRunnerFrame::createTestResult()

{

	return new TestResultWX( this  );

}







void wxTestRunnerFrame::addFailureError(TestResultDelegator* pResult, TestFailure *e, bool bIsError)

{

	const char* pszString = bIsError ? "ERROR: %s:%d: %s\n" : "FAIL:  %s:%d: %s\n" ;

	long nBefore = wxGetApp().m_pTestResults->GetInsertionPoint();

    //wxGetApp().m_pTestResults->SetForegroundColour(*wxRED);

	char buf[1024 * 8];

	sprintf( buf, pszString, e->getFileName(), e->getLineNumber(), e->getMessage() );

	wxGetApp().m_pTestResults->AppendText( buf );

	long nAfter = wxGetApp().m_pTestResults->GetInsertionPoint();

    wxGetApp().m_pTestResults->SetStyle( nBefore, nAfter-1, *wxRED);

	wxGetApp().m_pTestResults->ShowPosition( wxGetApp().m_pTestResults->GetLastPosition() );

	char szFileName[1024 * 2];

	pResult->getOutputFileName( e->getTest(), szFileName, false ) ;

	if ( *szFileName ) {

		strcat(szFileName, "stderr.txt");

		FILE* f = fopen( szFileName, "a" );

		if ( f ) {

			fputs( buf, f);

			fclose( f );

		}

	}

}



void wxTestRunnerFrame::addError(TestResultDelegator* pResult, TestFailure *e)

{

	TestRunner::addError( pResult, e );

	addFailureError( pResult, e, true );

}



void wxTestRunnerFrame::addFailure(TestResultDelegator* pResult, TestFailure *e)

{

	TestRunner::addFailure( pResult, e );

	addFailureError( pResult, e, false );

}





void wxTestRunnerFrame::startTest(TestResultDelegator* pResult, Test * pTest )

{

	TestRunner::startTest( pResult, pTest );



	char buf[ 1024 * 2];



	// nuke the old stderr.txt and stdout.txt

	pResult->getOutputFileName( pTest, buf, false ) ;

	if ( *buf ) 

	{

		char *pEnd = buf + strlen(buf);

		strcpy( pEnd, "stderr.txt" );

		wxRemoveFile( buf );

		strcpy( pEnd, "stdout.txt" );

		wxRemoveFile( buf );

	}



	wxGetApp().m_pTestResults->Clear();

	m_pTestTree->selectTest( pTest );

}



void wxTestRunnerFrame::appendTextFromFile( const char* pszFileName )

{

    wxStructStat st;

	if ( !::wxStat( pszFileName, &st) && st.st_size )

	{

		FILE* pFile = fopen( pszFileName, "rb" );

		if ( pFile ) 

		{

			char * p = new char[ st.st_size + 1 ];

			if ( fread( p, 1, st.st_size, pFile ) == (size_t)st.st_size ) 

			{

				p[ st.st_size ] = 0;

#ifdef __WXMAC__

				std::replace( p, p + st.st_size, '\r', '\n' );  // mac is fucked

#endif

				wxGetApp().m_pTestResults->AppendText( p );

			}

			delete [] p;

			fclose( pFile );

		}

	}

}



void appendToFile( const char* pszFileName, const char* szText )

{

	FILE* pFile = fopen( pszFileName, "a" );

	if ( pFile )

	{

		fputs( szText, pFile );

		fclose( pFile );

	}

}



bool loadFile( const char* pszPathName, std::vector< wxByte >& output )

{

	// read the file size

	struct stat statBuff;

	if ( !stat( pszPathName, &statBuff ) && statBuff.st_size ) {

		// open the file for binary read

		FILE *f = fopen( pszPathName, "rb" );

		if (f) {

			// read the file into memory

			output.clear();

			output.reserve( statBuff.st_size + 1 );

			output.resize( statBuff.st_size );

			int n = fread( output.begin(), statBuff.st_size, 1, f );

			if ( !fclose(f) && n == 1)

				return true;

		}

	}

	output.clear();

	return false;

}



#if defined __WXMSW__

#define EOL "\r\n"

#elif defined __WXMAC__

#define EOL "\r"

#else

#define EOL "\n"

#endif





void wxTestRunnerFrame::endTest(TestResultDelegator* pResult, Test * pTest)

{



	TestRunner::endTest( pResult, pTest );

	::wxLogMessage( "%s complete:  %s", pTest->toString(), pTest->getState() == Test::pass ? "PASS" : "FAIL"  );



	int nNumSubTests = pTest->getNumSubTests();

	if ( nNumSubTests )

	{

		// a test suite just ended

		// glue together all the outputs from the sub tests

		Test** pSubTests = (Test**)alloca( sizeof(Test*) * nNumSubTests );

		pTest->getSubTests( pSubTests );



		// get the path for the test suite

		char szTestPath[1024 * 2];

		getOutputFileName( pTest, szTestPath, false );

		char *pTestPathEnd = szTestPath + strlen( szTestPath );



		// nuke the existing outputs for the test suite

		wxFile wxf;

		strcpy( pTestPathEnd, "stdout.txt" );

		wxRemoveFile( szTestPath );				// nuke old file

		strcpy( pTestPathEnd, "stderr.txt" );

		wxRemoveFile( szTestPath );				// nuke old file



		std::vector< std::vector< wxByte> > filesStdOut, filesStdErr;

		filesStdOut.reserve( nNumSubTests );

		filesStdErr.reserve( nNumSubTests );

		filesStdOut.resize( nNumSubTests );

		filesStdErr.resize( nNumSubTests );

		bool bAtLeastOneOut = false;

		bool bAtLeastOneErr = false;

		int nTest;

		for ( nTest = 0; nTest < nNumSubTests; ++nTest )

		{

			// get the sub test name

			char szSubTestPath[1024 * 2];

			getOutputFileName( pSubTests[ nTest ], szSubTestPath, false );

			char *pSubTestPathEnd = szSubTestPath + strlen( szSubTestPath );



			strcpy( pSubTestPathEnd, "stdout.txt");

			if ( loadFile( szSubTestPath, filesStdOut[ nTest ] ) && filesStdOut[ nTest ].size() )

			{

				bAtLeastOneOut = true;

				filesStdOut[ nTest ].push_back(0);

			}



			strcpy( pSubTestPathEnd, "stderr.txt");

			if ( loadFile( szSubTestPath, filesStdErr[ nTest ] ) && filesStdErr[ nTest ].size() )

			{

				bAtLeastOneErr = true;

				filesStdErr[ nTest ].push_back(0);

			}



		}



		// iterate through all the tests and do the gluing

		strcpy( pTestPathEnd, "stdout.txt");

		FILE* pOut = bAtLeastOneOut ? fopen( szTestPath, "wb" ) : NULL;

		strcpy( pTestPathEnd, "stderr.txt");

		FILE* pErr = bAtLeastOneErr ? fopen( szTestPath, "wb" ) : NULL;

		for ( nTest = 0; nTest < nNumSubTests; ++nTest )

		{

			if ( filesStdOut[ nTest ].size() && pOut )

			{

				if ( !pSubTests[nTest]->getNumSubTests() ) // don't print if this isn't a test case

					fprintf( pOut, "<%s>" EOL, getFullTestName( pSubTests[nTest] ).c_str() );

				fprintf( pOut, "%s", filesStdOut[ nTest ].begin());

				if ( filesStdOut[ nTest ].size() && filesStdOut[ nTest ].size() )

					fprintf( pOut, EOL);

			}

			if ( filesStdErr[ nTest ].size() && pErr)

			{

				if ( !pSubTests[nTest]->getNumSubTests() ) // don't print if this isn't a test case

					fprintf( pErr, "<%s>" EOL, getFullTestName( pSubTests[nTest] ).c_str() );

				fprintf( pErr, "%s", filesStdErr[ nTest ].begin() );

				if ( !pSubTests[nTest]->getNumSubTests() && filesStdErr[ nTest ].size() ) // don't print if this isn't a test case

					fprintf( pErr, EOL);

			}

		}

		if ( pOut )

			fclose( pOut );

		if ( pErr )

			fclose( pErr );

	}

	else 

	{

		if (  pTest->getState() == Test::pass )

		{

			char szpathBuf[1024 * 2], szConfirmedPath[ 1024 * 2];

			*szpathBuf=0;

			pResult->getOutputFileName( pTest, szpathBuf, false );

			if ( *szpathBuf ) 

			{

				strcpy( szConfirmedPath, szpathBuf);

				strcat( szConfirmedPath, "confirmed.txt" );

				strcat( szpathBuf, "stdout.txt" );



				appendTextFromFile( szpathBuf );

				wxGetApp().m_pTestResults->SetInsertionPoint(0);



				// see if confirmed output file exists

				wxStructStat stConfirmed, stNew;

				bool bConfirmedExists = !::wxStat( szConfirmedPath, &stConfirmed);

				bool bNewExists       = !::wxStat( szpathBuf, &stNew);



				if ( bConfirmedExists )

				{

					bool bOutputsSame = true;

					if ( !bNewExists ) 

					{

						bOutputsSame = false; // failure -- used to be output but now there's none.

					}

					else 

					{

						if ( stConfirmed.st_size != stNew.st_size )

						{

							bOutputsSame = false; // failure--output's are different size

						}

						else 

						{

							char * pNewOutput = new char[ stNew.st_size + 1 ];

							char * pConfirmedOutput = new char[ stConfirmed.st_size + 1 ];

							FILE* pFile = fopen( szpathBuf, "rb" );

							if ( pFile )

							{

								fread( pNewOutput, 1, stNew.st_size, pFile ); 

								fclose( pFile );

							}



							pFile = fopen( szConfirmedPath, "rb" );

							if ( pFile )

							{

								fread( pConfirmedOutput, 1, stConfirmed.st_size, pFile ); 

								fclose( pFile );

							}



							if ( memcmp( pConfirmedOutput, pNewOutput, stNew.st_size ) != 0 )

							{

								// failure--outputs are same size but different outputs

								bOutputsSame = false;

							}

							delete [] pConfirmedOutput;

							delete [] pNewOutput;

						}



						if ( !bOutputsSame )

						{

							pTest->setState( Test::fail );



							// write in stderr.txt that output differs

							char szStdOutPathBuf[1024 * 2];

							getOutputFileName( pTest, szStdOutPathBuf, false );

							strcat( szStdOutPathBuf, "stderr.txt" );

							FILE* fErr = fopen( szStdOutPathBuf, "w" );

							if ( fErr )

							{

								fprintf( fErr, "Output differs from last confirmed output.\n" );

					l			fclose( fErr );

								onTestSelected( pTest ); // this updates the display with the above text

							}

						}

					}

				}

				else 

				{

					if ( bNewExists && stNew.st_size )

					{



						if( DoConfirmResultsDlg() == wxID_YES ) 

						{

							if ( !wxCopyFile( szpathBuf, szConfirmedPath) )

							{

								wxLogError( "Could not write output to file:  %s", szConfirmedPath );

							}

						}

						else 

						{

							pTest->setState( Test::fail );

						}

					}

				}

			}

		}

	}



	m_pTestTree->onTestFinished( pTest );



	if ( pTest == m_pSuiteTests )

	{

		m_pTestTree->onTestFinished( pTest );

		m_pTestTree->SelectItem( m_pTestTree->GetRootItem() );

	}

}



void wxTestRunnerFrame::runTest( Test* pTest )

{

	TestRunner::runTest( pTest );

	m_pTestTree->Unselect();

}



void wxTestRunnerFrame::onTestSelected( Test* pTest )

{

	// clear out old contents

	wxGetApp().m_pTestResults->Clear();



	// get the output dir

	char szFName[1024 * 2];

	TestResultWX( this ).getOutputFileName( pTest, szFName, false );

	if ( *szFName) 

	{

		// stuff in any error text

		char *pEnd = szFName + strlen( szFName );

		strcpy( pEnd, "stderr.txt" );

		long nBefore = wxGetApp().m_pTestResults->GetInsertionPoint();

		appendTextFromFile( szFName );

		long nAfter = wxGetApp().m_pTestResults->GetInsertionPoint();

		if ( nBefore != nAfter )

			wxGetApp().m_pTestResults->SetStyle( nBefore, nAfter-1, *wxRED);



		strcpy( pEnd, "stdout.txt" );

		appendTextFromFile( szFName );

		wxGetApp().m_pTestResults->ShowPosition(0);

	}

}

*/



void wxTestRunnerFrame::OnSetOutputDir(wxCommandEvent& event)

{

	wxDirDialog dlg( this, "Select Output Subdirectory", wxGetApp().GetOutputSubDir() );

    if (dlg.ShowModal() == wxID_OK)

    {

		wxString strOutputPath = dlg.GetPath();

		if ( strOutputPath.Length() && strOutputPath[ strOutputPath.Length()-1 ] != wxFILE_SEP_PATH )

			strOutputPath += wxFILE_SEP_PATH;



		wxGetApp().SetOutputSubDir( strOutputPath );

	}

}

