/////////////////////////////////////////////////////////////////////////////

// Name:        wxTestRunnerGUIApp.cpp

// Purpose:     

// Author:      Marty Saxton

// Modified by:

// Created:     10/27/2001

// RCS-ID:      $Id: wxTestRunnerGUIApp.cpp,v 1.1.1.1 2001/11/10 19:00:46 thundercow Exp $

// Copyright:   (c) Marty Saxton, Thundercow Software

// Licence:     wxWindows license

/////////////////////////////////////////////////////////////////////////////



#include "PrecompiledHeaders.h"



#include "wxTestTree.h"

#include "wxTestRunnerGUIApp.h"

#include <wx/splitter.h>



#include <malloc.h>



#if defined(__WXGTK__) || defined(__WXMOTIF__) || defined ( __WXMAC__ )

    #include "../../unix/wxTestRunnerGUI.xpm"

#endif



#include "wxTestRunnerFrame.h"





template <class Runner>

class GUIRunner : public Runner

{

public:

	GUIRunner( wxTestRunnerGUIApp* pApp ) : m_pApp(  pApp ) {}

/*

	virtual void OnFoundTest( wxTestModule& testModule ) 

	{ 

		Runner::OnFoundTest( testModule );

		//m_pApp->OnFoundTest( testModule ); 

	}

*/



	virtual void OnTestAdded( wxITest* pTest )

	{

		Runner::OnTestAdded( pTest );

		m_pApp->AddTest( pTest );			// updates the GUI

	}



	virtual void BeginTestCase( wxITest* pTest )

	{

		Runner::BeginTestCase( pTest );

		m_pApp->BeginTestCase( pTest );

	}



	virtual void EndTestCase( wxITest* pTest )

	{

		Runner::EndTestCase( pTest );

		m_pApp->EndTestCase( pTest );

	}



	virtual void OnTestOutput( wxITest* pTest, const char* pszOutput )

	{

		wxTestRunner::OnTestOutput( pTest, pszOutput );

		m_pApp->OnTestOutput( pTest, pszOutput );



	}



	virtual void OnTestFailure( wxITest* pTest, const char* pszMessage, const char* pszFile, unsigned long nLineNo )

	{

		wxTestRunner::OnTestFailure( pTest, pszMessage, pszFile, nLineNo );

		m_pApp->OnTestFailure( pTest, pszMessage, pszFile, nLineNo );





	}





private:

	wxTestRunnerGUIApp * const m_pApp;

};





// IDs for the controls and the menu commands

enum

{

    // menu items

    Minimal_Quit = 1,

    Minimal_About

};



IMPLEMENT_DYNAMIC_CLASS( wxTestOutputEvent, wxEvent );



IMPLEMENT_APP(wxTestRunnerGUIApp)



DEFINE_EVENT_TYPE(wxEVT_TEST_OUTPUT);





BEGIN_EVENT_TABLE(wxTestRunnerGUIApp, wxApp)

	EVT_TEST_OUTPUT(wxTestRunnerGUIApp::OnTestOutputEvent)

END_EVENT_TABLE()





// ============================================================================

// implementation

// ============================================================================



// ----------------------------------------------------------------------------

// the application class

// ----------------------------------------------------------------------------





wxString GetAppDir()

{

#ifdef __WXMSW__

	char szDir[ _MAX_PATH ], szDrive[ _MAX_PATH ], szPath[ _MAX_PATH];

	DWORD dwResult = GetModuleFileName( NULL, szDir, sizeof( szDir ) ) ;

	if ( dwResult ) {

		_splitpath( szDir, szDrive, szPath, NULL, NULL );

		strcat( szDrive, szPath);

		return szDrive;

	}

#endif

	return wxGetCwd();

}





wxTestRunnerGUIApp::wxTestRunnerGUIApp() : m_config( "wxTest", "ThunderCowSoftware", GetAppDir() + "wxTestConfig.txt", "", wxCONFIG_USE_LOCAL_FILE )

{

#ifdef WXTEST_MULTITHREADED

	m_pRunner = new GUIRunner< wxTestRunnerMT >( this );

#else

	m_pRunner = new GUIRunner< wxTestRunner >( this );

#endif

	

	wxString strOutputDir = m_config.Read( "OutputSubDir", GetAppDir() );

	m_pRunner->SetOutputSubDir( strOutputDir );

}





wxTestRunnerGUIApp::~wxTestRunnerGUIApp()

{

	delete m_pRunner;

}



// 'Main program' equivalent: the program execution "starts" here

bool wxTestRunnerGUIApp::OnInit()

{

    // create the main application window

    wxFrame *frame = new wxTestRunnerFrame("Minimal wxWindows App",

                                 wxPoint(50, 50), wxSize(450, 340));



    // and show it (the frames, unlike simple controls, are not shown when

    // created initially)

    frame->Show(TRUE);



	m_pRunner->LoadTests();



    // success: wxApp::OnRun() will be called which will enter the main message

    // loop and the application will run. If we returned FALSE here, the

    // application would exit immediately.

    return TRUE;

}



/*

void wxTestRunnerGUIApp::OnFoundTest( wxTestModule& testModule )

{

	m_pTestTree->AddTestsToTree( m_pTestTree->GetRootItem(), testModule.m_pTest, testModule.m_pTest->GetDescription() );

}

*/



void wxTestRunnerGUIApp::AddTest( wxITest* pTest )

{

	m_pTestTree->AddTestsToTree( pTest, pTest->GetDescription() );

}





void wxTestRunnerGUIApp::RunTest( wxITest* pTest )

{

	pTest->Run( m_pRunner );

}





void wxTestRunnerGUIApp::BeginTestCase( wxITest* pTest )

{

	m_pTestTree->UpdateTest( pTest );

	m_pTestResults->Clear();

}





void wxTestRunnerGUIApp::EndTestCase( wxITest* pTest )

{

	//m_pTestTree->UpdateTest( pTest );



	if ( pTest->GetState() == wxITest::pass || pTest->GetState() == wxITest::running  )

	{

		char szPathOutput[1024], szPathConfirmed[1024];

		m_pRunner->GetOutputSubdir( pTest, szPathOutput, true );

		strcpy( szPathConfirmed, szPathOutput );

		

		char * pEnd = szPathOutput + strlen( szPathOutput );

		*pEnd++ = wxFILE_SEP_PATH;

		strcpy (pEnd, "output.txt" );



		pEnd = szPathConfirmed + strlen( szPathConfirmed );

		*pEnd++ = wxFILE_SEP_PATH;

		strcpy (pEnd, "confirmed.txt" );





		if ( wxFile::Exists(szPathOutput) ) 

		{



			if ( !wxFile::Exists(szPathConfirmed) ) 

			{

				int DoConfirmResultsDlg();  // prototpye 

				if ( DoConfirmResultsDlg() == wxNO ) 

				{

					pTest->SetState( wxITest::fail );

				}

				else 

				{

					pTest->SetState( wxITest::pass );

					wxRenameFile( szPathOutput, szPathConfirmed );

				}

			}

			else 

			{

				// both files exist, compare them

				if ( !CompareFiles( szPathOutput, szPathConfirmed ) )

				{

					pTest->SetState( wxITest::fail );

				}

			}

		}

	}

	m_pTestTree->UpdateTest( pTest );

}



wxITest* wxTestRunnerGUIApp::GetTestRoot()

{

	return m_pRunner->GetTestRoot();

}



void wxTestRunnerGUIApp::OnTestOutput( wxITest* pTest, const char* pszOutput )

{

	m_pTestResults->AppendText( pszOutput );

	

	ProcessThreadEvent( wxTestOutputEvent( pTest)  );



}





wxString wxTestRunnerGUIApp::GetOutputSubDir() const

{

	return m_pRunner->GetOutputSubdir();

}



void wxTestRunnerGUIApp::SetOutputSubDir( wxString strOutputDir )

{

	m_pRunner->SetOutputSubDir( strOutputDir );

	m_config.Write( "OutputSubDir", strOutputDir );

	m_config.Flush();

}



void wxTestRunnerGUIApp::OnTestSelected( wxITest* pTest )

{

	m_pTestResults->Clear();

	

	char szOutputPathName[1024 * 4 ];

	m_pRunner->GetOutputSubdir( pTest, szOutputPathName, false );

	char szSep[] = { wxFILE_SEP_PATH, 0 };

	char *pEnd = szOutputPathName + strlen( szOutputPathName );

	*pEnd++ = wxFILE_SEP_PATH;

	strcpy( pEnd, "failures.txt" );

	if ( !wxFile::Exists( szOutputPathName ) )

		strcpy( pEnd, "output.txt" );

	if ( wxFile::Exists( szOutputPathName ) )

	{

		wxFile file( szOutputPathName );

		if ( file.IsOpened() )

		{

			int nLen = file.Length();

			char * buf = (char*)alloca( nLen + 1 );

			file.Read( buf, nLen );

			buf[ nLen ] = 0;

			m_pTestResults->AppendText( buf );

		}

	}

}



void wxTestRunnerGUIApp::OnTestOutputEvent( wxTestOutputEvent& evt)

{

	wxITest* pTest = evt.GetTest();

}



bool wxTestRunnerGUIApp::CompareFiles( const char* str1, const char* str2 )

{

	wxFile file1, file2;

	file1.Open( str1 );

	file2.Open( str2 );



	if ( file1.IsOpened() != file2.IsOpened() )

		return false;

	if ( !file1.IsOpened() && !file2.IsOpened() )

		return true;



	if ( file1.Length() != file2.Length() )

		return false;



	int nLen = file1.Length();

	char* buf1 = new char[ nLen ];

	char* buf2 = new char[ nLen ];



	file1.Read( buf1, nLen );

	file2.Read( buf2, nLen );

	return memcmp( buf1, buf2, nLen ) ? false : true;

}





void wxTestRunnerGUIApp::OnTestFailure( wxITest* pTest, const char* pszMessage, const char* pszFile, unsigned long nLineNo )

{

	char buf[1024 * 4];

	sprintf(buf, "FAILURE:  %s:%d %s\n", pszFile, nLineNo, pszMessage );

	m_pTestResults->AppendText( buf );

}

