/////////////////////////////////////////////////////////////////////////////

// Name:        wxTestTree.cpp

// Purpose:     

// Author:      Marty Saxton

// Modified by:

// Created:     10/27/2001

// RCS-ID:      $Id: wxTestTree.cpp,v 1.1.1.1 2001/11/10 19:00:46 thundercow Exp $

// Copyright:   (c) Marty Saxton, Thundercow Software

// Licence:     wxWindows license

/////////////////////////////////////////////////////////////////////////////



#include "PrecompiledHeaders.h"



#include "wxTestTree.h"

#include "wxTestRunnerGUIApp.h"

#ifdef __WXMSW__

#pragma warning (disable : 4786 )

#include <windows.h>

#include <malloc.h>

#else

#include <alloca.h>

#endif



#include <wxTest/ITest.h>



#include "wxTestRunnerFrame.h"

#include <wx/imaglist.h>



#if defined(__WXGTK__) || defined(__WXMOTIF__) || defined ( __WXMAC__ )

    #include "../../unix/good.xpm"

    #include "../../unix/bad.xpm"

    #include "../../unix/running.xpm"

    #include "../../unix/unknown.xpm"

#endif





BEGIN_EVENT_TABLE(wxTestTree, wxTreeCtrl)

    EVT_LEFT_DCLICK(wxTestTree::OnLMouseDClick)

    EVT_TREE_SEL_CHANGED(WINDOWID_TEST_TREE, wxTestTree::OnSelChanged)

END_EVENT_TABLE()





wxTestTree::wxTestTree( wxWindow* pParent, wxWindowID id  ) :

	wxTreeCtrl(pParent, id , wxDefaultPosition, wxDefaultSize, wxTR_HAS_BUTTONS | wxTR_LINES_AT_ROOT )



{

	AddRoot( "all tests", -1, -1, new wxTestRunnerTreeItemData( wxGetApp().GetTestRoot() ) );

	m_mapTestToTreeId[ wxGetApp().GetTestRoot() ] = GetRootItem();



    wxImageList * pImageList = new wxImageList(16, 16, TRUE);

	pImageList->Add( wxICON( ICON_UNKNOWN ) );

	pImageList->Add( wxICON( ICON_GOOD    ) );

	pImageList->Add( wxICON( ICON_BAD     ) );

	pImageList->Add( wxICON( ICON_RUNNING ) );

    AssignImageList(pImageList);

}





void wxTestTree::OnLMouseDClick(wxMouseEvent& event)

{

	wxArrayTreeItemIds selectedItems;

    size_t nNumSelections = GetSelections( selectedItems );

	for ( size_t nSel = 0; nSel < nNumSelections; ++nSel ) {

		wxTreeItemId idSelected = selectedItems[ nSel ];

		wxTestRunnerTreeItemData* pData = static_cast< wxTestRunnerTreeItemData* >( GetItemData( idSelected ) );

		assert( pData );

		wxGetApp().RunTest( pData->m_pTest );

	}

}



void wxTestTree::OnSelChanged(wxTreeEvent& event)

{

	wxTreeItemId id = event.GetItem();

	if ( id.IsOk() )

	{

		wxTestRunnerTreeItemData* pData = static_cast< wxTestRunnerTreeItemData* >( GetItemData( event.GetItem() ) );

		if ( pData ) 

		{

			wxGetApp().OnTestSelected( pData->m_pTest );

		}

	}

    event.Skip();

}





void wxTestTree::selectTest( wxITest* pTest )

{

	MapTestToTreeId::iterator iter = m_mapTestToTreeId.find( pTest );

	if ( iter != m_mapTestToTreeId.end() ) 

	{

		SelectItem( (*iter).second );

	}



}







void wxTestTree::AddTestsToTree( wxITest* pTest, const char* pszLabel, wxTreeItemId parentId )

{

	if ( parentId == wxTreeItemId() )

	{

		MapTestToTreeId::iterator iter = m_mapTestToTreeId.find( pTest->GetParent() );

		if ( iter != m_mapTestToTreeId.end() )

		{

			parentId = (*iter).second;

		}

	}



	wxTreeItemId id = AppendItem(parentId, 

		pszLabel,

		0,

		-1,

		new wxTestRunnerTreeItemData( pTest ) );



	m_mapTestToTreeId[ pTest ] = id;





	if ( !IsExpanded( id ) /*&& parentId == GetRootItem()*/ )

		Expand( parentId );



	int nNumSubTests = pTest->GetNumSubTests();

	wxITest** subTests = (wxITest**)alloca( nNumSubTests * sizeof(wxITest*) );

	pTest->GetSubTests( subTests);

	for ( int i = 0;  i < nNumSubTests; ++i ) 

	{

		AddTestsToTree( subTests[i], subTests[i]->GetDescription(), id );

	}

}



void wxTestTree::onTestFinished( wxITest* pTest )

{

	MapTestToTreeId::iterator iter = m_mapTestToTreeId.find( pTest );

	if ( iter != m_mapTestToTreeId.end() ) 

	{

/*

		switch( pTest->GetState() )

		{

		case wxITest::unknown: nImage = 0; bkColor = wxWHITE; fgColor = wxBLACK; bBold = false; break;

		case wxITest::pass:    nImage = 1; bkColor = wxWHITE; fgColor = &wxDARKGREEN; bBold = false; break;

		case wxITest::fail:    nImage = 2; 

			if ( !pTest->GetNumSubTests() ) // color bk red if it's an actual test suite

			{

				bkColor = wxRED;   fgColor = wxWHITE; bBold = false;  break;

			}

			else

			{

				bkColor = wxWHITE;   fgColor = wxRED; bBold = false;  break;

			}

			

		}

		SetItemImage( (*iter).second, nImage, wxTreeItemIcon_Normal ); 

		SetItemImage( (*iter).second, nImage, wxTreeItemIcon_Selected ); 

    

	    SetItemBold( (*iter).second, bBold );

		SetItemTextColour( (*iter).second, *fgColor );

		SetItemBackgroundColour( (*iter).second, *bkColor);

*/



		// if test passed, collapse the tree

		if ( pTest->GetState() == wxITest::pass )

		{

			Collapse( (*iter).second );

		}

    

		pTest = pTest->GetParent();

		if ( pTest )

			onTestFinished( pTest );





	}



}



void wxTestTree::UpdateTest( wxITest* pTest )

{

	int nImage;

	wxColour * bkColor = wxWHITE, * fgColor = wxBLACK;

	bool bBold;

	wxColour wxDARKGREEN( 0,128,0 );



	MapTestToTreeId::iterator iter = m_mapTestToTreeId.find( pTest );

	if ( iter != m_mapTestToTreeId.end() ) 

	{

		switch( pTest->GetState() )

		{

			case wxITest::unknown: nImage = 0; bkColor = wxWHITE; fgColor = wxBLACK; bBold = false; break;

			case wxITest::pass:    nImage = 1; bkColor = wxWHITE; fgColor = &wxDARKGREEN; bBold = false; break;

			case wxITest::fail:    nImage = 2; break;

			case wxITest::running: nImage = 3; break;



		}

		SetItemImage( (*iter).second, nImage, wxTreeItemIcon_Normal ); 

		SetItemImage( (*iter).second, nImage, wxTreeItemIcon_Selected ); 



		SetItemBold( (*iter).second, bBold );

		SetItemTextColour( (*iter).second, *fgColor );

		SetItemBackgroundColour( (*iter).second, *bkColor);

	}

}



